"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports._onScroll = exports._toolPreview = exports._doStroke = exports._endStroke = exports._startStroke = exports.LineDefaults = exports.name = void 0;
var circle_1 = require("../../../pathMakers/circle");
var getRGBColorString_1 = require("../../../util/getRGBColorString");
var _path_1 = require("../../RealRenderer/strokeNodes/_path");
var line_1 = require("../../../pathMakers/line");
var _group_1 = require("../../RealRenderer/strokeNodes/_group");
var getRadiusFromThickness_1 = require("./util/getRadiusFromThickness");
exports.name = 'line';
exports.LineDefaults = {
    lineThickness: 1,
    lineColor: [1, 1, 1]
};
/** key -> identifier, value -> coordinate
 *  For mouse, the key is 'mouse', for touches, stringified identifier -> https://developer.mozilla.org/en-US/docs/Web/API/Touch/identifier
 */
var _startCoords = new Map(); /* key -> identifier, value -> coordinate*/
function _startStroke(coords, identifier) {
    this._doPreview = false;
    if (this._previewStroke.has(identifier)) {
        this._previewStroke.get(identifier).forEach(function (strokeNode) {
            strokeNode.delete();
        });
    }
    var linePath = new _path_1.Path('', 'strokes');
    linePath.setStroke(getRGBColorString_1.getRGBColorString(this.toolSettings.lineColor));
    linePath.setStrokeWidth(this.toolSettings.lineThickness);
    var startCircle = circle_1.getCircleNode(coords, getRadiusFromThickness_1.getRadiusFromThickness(this.toolSettings.lineThickness), this.toolSettings.lineColor, 'strokes');
    var endCircle = circle_1.getCircleNode(coords, getRadiusFromThickness_1.getRadiusFromThickness(this.toolSettings.lineThickness), this.toolSettings.lineColor, 'strokes');
    this._addStroke([new _group_1.GroupNode('strokes', [linePath, startCircle, endCircle])]);
    this._strokeIdentifierMap.set(identifier, this._strokeIndex);
    _startCoords.set(identifier, coords);
}
exports._startStroke = _startStroke;
function _endStroke(endCoords, identifier) {
    var lineNode = this.strokes[this._strokeIdentifierMap.get(identifier)][0];
    lineNode.innerNodes[0].updatePath(line_1.getLinePathCommand(_startCoords.get(identifier), endCoords));
    lineNode.innerNodes[2].updateCenter(endCoords);
    _startCoords.delete(identifier);
    this._doPreview = true;
}
exports._endStroke = _endStroke;
function _doStroke(coords, identifier) {
    var lineNode = this.strokes[this._strokeIndex][0];
    lineNode.innerNodes[0].updatePath(line_1.getLinePathCommand(_startCoords.get(identifier), coords));
    lineNode.innerNodes[2].updateCenter(coords);
}
exports._doStroke = _doStroke;
function _toolPreview(coords, identifier) {
    if (this._previewStroke.get(identifier).length == 0) {
        var circleNode = circle_1.getCircleNode(coords, getRadiusFromThickness_1.getRadiusFromThickness(this.toolSettings.lineThickness), this.toolSettings.lineColor, 'overlay');
        circleNode.setFill(getRGBColorString_1.getRGBColorString(this.toolSettings.lineColor));
        circleNode.setStroke(getRGBColorString_1.getRGBColorString(this.toolSettings.lineColor));
        this._previewStroke.get(identifier).push(circleNode);
    }
    else {
        var circleNode = this._previewStroke.get(identifier)[0];
        circleNode.updateCenter(coords);
        circleNode.updateRadius(getRadiusFromThickness_1.getRadiusFromThickness(this.toolSettings.lineThickness));
        circleNode.setFill(getRGBColorString_1.getRGBColorString(this.toolSettings.lineColor));
        circleNode.setStroke(getRGBColorString_1.getRGBColorString(this.toolSettings.lineColor));
    }
}
exports._toolPreview = _toolPreview;
function _onScroll(scrollDelta, coords, identifier) {
    this.changeToolSetting('lineThickness', Math.max(1, this.toolSettings.lineThickness - scrollDelta));
    if (this._previewStroke.get(identifier) && this._previewStroke.get(identifier).length !== 0) {
        this._previewStroke.get(identifier)[0].updateRadius(getRadiusFromThickness_1.getRadiusFromThickness(this.toolSettings.lineThickness));
        this._display(this._previewStroke.get(identifier));
    }
}
exports._onScroll = _onScroll;
