"use strict";
var __extends = (this && this.__extends) || (function () {
    var extendStatics = function (d, b) {
        extendStatics = Object.setPrototypeOf ||
            ({ __proto__: [] } instanceof Array && function (d, b) { d.__proto__ = b; }) ||
            function (d, b) { for (var p in b) if (Object.prototype.hasOwnProperty.call(b, p)) d[p] = b[p]; };
        return extendStatics(d, b);
    };
    return function (d, b) {
        extendStatics(d, b);
        function __() { this.constructor = d; }
        d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
    };
})();
var __assign = (this && this.__assign) || function () {
    __assign = Object.assign || function(t) {
        for (var s, i = 1, n = arguments.length; i < n; i++) {
            s = arguments[i];
            for (var p in s) if (Object.prototype.hasOwnProperty.call(s, p))
                t[p] = s[p];
        }
        return t;
    };
    return __assign.apply(this, arguments);
};
var __createBinding = (this && this.__createBinding) || (Object.create ? (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    Object.defineProperty(o, k2, { enumerable: true, get: function() { return m[k]; } });
}) : (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    o[k2] = m[k];
}));
var __exportStar = (this && this.__exportStar) || function(m, exports) {
    for (var p in m) if (p !== "default" && !Object.prototype.hasOwnProperty.call(exports, p)) __createBinding(exports, m, p);
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.RealRenderer = exports.RealRendererTypes = void 0;
var eventEmitter_1 = require("./events/eventEmitter");
var RealRendererEvents_1 = require("../../constants/events/RealRendererEvents");
var blankGraph_1 = require("../../pathMakers/blankGraph");
exports.RealRendererTypes = require("../../types/RealRendererTypes");
var RealRendererDefaults_1 = require("../../constants/defaults/RealRendererDefaults");
__exportStar(require("../../constants/defaults/RealRendererDefaults"), exports);
var undo_1 = require("./undo");
var _circle_1 = require("./strokeNodes/_circle");
var _path_1 = require("./strokeNodes/_path");
var _text_1 = require("./strokeNodes/_text");
var _polygon_1 = require("./strokeNodes/_polygon");
var clamp_1 = require("../../util/clamp");
/**
 * General Real Renderer with no specific purpose. Should be extended to use.
 */
var RealRenderer = /** @class */ (function (_super) {
    __extends(RealRenderer, _super);
    /**
     *
     * @param options
     * @param eventList
     */
    function RealRenderer(options, eventList) {
        if (eventList === void 0) { eventList = RealRendererEvents_1.RealRendererEventList; }
        var _this = _super.call(this, eventList) || this;
        _this._offsetX = 0;
        _this._offsetY = 0;
        _this.strokes = [];
        _this._strokeIndex = -1;
        _this.undo = undo_1.undo;
        _this.redo = undo_1.redo;
        // *****DEFAULTS*****
        _this.settings = __assign(__assign({}, RealRendererDefaults_1.RealRendererDefaults), options);
        switch (_this.settings.bgType.type) {
            case 'axes':
                _this.settings.bgType.xOffset = clamp_1.clamp(_this.settings.bgType.xOffset, 0, 100); // %age
                _this.settings.bgType.yOffset = clamp_1.clamp(_this.settings.bgType.yOffset, 0, 100); // %age
                break;
            case 'grid':
                _this.settings.bgType.xSpacing = clamp_1.clamp(_this.settings.bgType.xSpacing, 0, 100); // %age
                _this.settings.bgType.ySpacing = clamp_1.clamp(_this.settings.bgType.ySpacing, 0, 100); // %age
                if (_this.settings.bgType.xSpacing === 0)
                    _this.settings.bgType.xSpacing = 1;
                if (_this.settings.bgType.ySpacing === 0)
                    _this.settings.bgType.ySpacing = 1;
                break;
            case 'ruled':
                _this.settings.bgType.spacing = clamp_1.clamp(_this.settings.bgType.spacing, 0, 100);
                if (_this.settings.bgType.spacing === 0)
                    _this.settings.bgType.spacing = 1;
                break;
            case 'none':
                break;
            default:
                break;
        }
        _this.bgColor = _this.settings.bgColor;
        _this.bgType = _this.settings.bgType;
        _this.drawsPerFrame = _this.settings.drawsPerFrame;
        _this.timeStep = _this.settings.timeStep;
        _this.time = _this.settings.initTime;
        _this.scaleFactor = _this.settings.scaleFactor;
        return _this;
        // *****DEFAULTS*****
    }
    RealRenderer.prototype._setSVG = function () {
        if (this.svg === undefined) {
            throw 'No SVG Element Found';
        }
        this._setViewBox(this.dimensions, this._offsetX, this._offsetY);
        this.svg.setAttribute('preserveAspectRatio', 'none');
        this.svgSections = {
            bg: document.createElementNS('http://www.w3.org/2000/svg', 'g'),
            strokes: document.createElementNS('http://www.w3.org/2000/svg', 'g'),
            overlay: document.createElementNS('http://www.w3.org/2000/svg', 'g')
        };
        this.svg.appendChild(this.svgSections.bg);
        this.svg.appendChild(this.svgSections.strokes);
        this.svg.appendChild(this.svgSections.overlay);
        this._addStroke(blankGraph_1.getBlankGraphPaths(this.dimensions, this.bgColor, this.bgType));
        this._display(this.strokes[this._strokeIndex]);
        this._doRender = false;
    };
    RealRenderer.prototype._setViewBox = function (dimensions, xOffset, yOffset) {
        this.svg.setAttribute('viewBox', xOffset + " " + yOffset + " " + dimensions[0] + " " + dimensions[1]);
    };
    RealRenderer.prototype._drawFunc = function (time) {
        return this;
    };
    RealRenderer.prototype._draw = function () {
        this.time += this.timeStep;
        this._drawFunc(this.time);
        return this;
    };
    RealRenderer.prototype._addStroke = function (stroke) {
        if (this.strokes.length > this._strokeIndex + 1)
            this.strokes.splice(this._strokeIndex + 1, this.strokes.length - this._strokeIndex);
        this.strokes[++this._strokeIndex] = stroke;
    };
    RealRenderer.prototype._render = function () {
        var _this = this;
        if (this._doRender) {
            this.draw(this.drawsPerFrame);
            this._display(this.strokes[this._strokeIndex]);
            window.requestAnimationFrame(function () { _this._render(); });
        }
    };
    RealRenderer.prototype._display = function (stroke) {
        var _this = this;
        stroke.forEach(function (strokeNode) {
            if (strokeNode.node.parentElement == null)
                _this.svgSections[strokeNode.section].appendChild(strokeNode.node);
        });
    };
    /**
     * Attaches to a DOM SVG element to render to.
     * @param svg The SVG element to attach.
     * @param dimensions Dimensions of the graph.
     * @returns Self for chaining.
     */
    RealRenderer.prototype.attach = function (svg, dimensions) {
        this.dimensions = dimensions;
        this.originalDimensions = [
            dimensions[0],
            dimensions[1]
        ];
        this.svg = svg;
        this._setSVG();
        return this;
    };
    /**
     * Start rendering.
     * @returns Self for chaining.
     */
    RealRenderer.prototype.startRender = function () {
        if (!this._doRender) {
            this._doRender = true;
            this._render();
            this.emit('start-render', {});
        }
        return this;
    };
    /**
     * Stop rendering.
     * @returns Self for chaining.
     */
    RealRenderer.prototype.stopRender = function () {
        this._doRender = false;
        this.emit('stop-render', {});
        return this;
    };
    /**
     * Toggle rendering.
     * @returns Self for chaining.
     */
    RealRenderer.prototype.toggleRender = function () {
        this._doRender = !this._doRender;
        if (this._doRender)
            this._render();
        return this;
    };
    /**
     * Draw a certain number of frames.
     * @param numDraws Number of frames to draw.
     * @returns Self for chaining.
     */
    RealRenderer.prototype.draw = function (numDraws) {
        if (numDraws === void 0) { numDraws = 1; }
        for (var i = 0; i < numDraws; i++)
            this._draw();
        return this;
    };
    /**
     * Scale/zoom the graph.
     * @param scaleFactor Factor to scale the graph by. Larger number zooms in.
     * @returns Self for chaining.
     */
    RealRenderer.prototype.scale = function (scaleFactor) {
        var oldScaleFactor = this.scaleFactor;
        this.scaleFactor = scaleFactor;
        this.dimensions[0] = this.originalDimensions[0] / scaleFactor;
        this.dimensions[1] = this.originalDimensions[1] / scaleFactor;
        // To clamp the offsets as well
        this.changeOffsets(this._offsetX, this._offsetY);
        this.emit('change-scale', {
            oldScaleFactor: oldScaleFactor,
            newScaleFactor: this.scaleFactor
        });
        return this;
    };
    /**
     * Change the offsets of the graph (for panning)
     * @param xOffset Offset in the x-direction.
     * @param yOffset Offset in the y-direction.
     * @returns Self for chaining.
     */
    RealRenderer.prototype.changeOffsets = function (xOffset, yOffset) {
        var oldOffsets = {
            x: this._offsetX,
            y: this._offsetY
        };
        this._offsetX = clamp_1.clamp(xOffset, 0, this.originalDimensions[0] - this.dimensions[0]);
        this._offsetY = clamp_1.clamp(yOffset, 0, this.originalDimensions[1] - this.dimensions[1]);
        this._setViewBox(this.dimensions, this._offsetX, this._offsetY);
        this.emit('change-offsets', {
            oldOffsets: oldOffsets,
            newOffsets: {
                x: this._offsetX,
                y: this._offsetY
            }
        });
        return this;
    };
    /**
     * Export the data of the graph in a certain format that can be used to load the data later. Load using .importData().
     * @returns Data of the graph in a storable and loadable format.
     */
    RealRenderer.prototype.exportData = function () {
        var strokeExport = [];
        this.strokes.forEach(function (stroke) {
            strokeExport.push(stroke.map(function (node) { return node.export(); }));
        });
        return {
            exportData: strokeExport,
            strokeIndex: this._strokeIndex,
            dimensions: this.dimensions,
            bgType: this.bgType
        };
    };
    /**
     * Import the data exported by .export() method.
     * @param data Data exported by .export()
     * @returns Self for chaining.
     */
    RealRenderer.prototype.importData = function (data) {
        var _this = this;
        this.strokes.forEach(function (stroke) {
            stroke.forEach(function (node) { return node.delete(); });
        });
        this.svg.setAttribute('viewBox', "0 0 " + this.dimensions[0] + " " + this.dimensions[1]);
        if (data.bgType)
            this.bgType = data.bgType;
        this.strokes = [];
        data.exportData.forEach(function (strokeExport) {
            _this.strokes.push(strokeExport.map(function (strokeNodeData) {
                switch (strokeNodeData.type) {
                    case 'circle':
                        var circ = new _circle_1.Circle([0, 0], 0, strokeNodeData.section ? strokeNodeData.section : 'strokes');
                        circ.import(strokeNodeData.data);
                        return circ;
                    case 'path':
                        var path = new _path_1.Path('', strokeNodeData.section ? strokeNodeData.section : 'strokes');
                        path.import(strokeNodeData.data);
                        return path;
                    case 'text':
                        var text = new _text_1.Text([0, 0], '', strokeNodeData.section ? strokeNodeData.section : 'strokes');
                        text.import(strokeNodeData.data);
                        return text;
                    case 'polygon':
                        var polygon = new _polygon_1.Polygon([], strokeNodeData.section ? strokeNodeData.section : 'strokes');
                        polygon.import(strokeNodeData.data);
                        return polygon;
                }
            }));
        });
        this._strokeIndex = data.strokeIndex;
        for (var i = 0; i <= this._strokeIndex; i++) {
            this._display(this.strokes[i]);
        }
        this.emit('import', {
            import: data
        });
        return this;
    };
    /**
     * Resets the internal time counter.
     * @returns Self for chaining.
     */
    RealRenderer.prototype.resetTime = function () {
        this.time = 0;
        return this;
    };
    /**
     * Resets everything regarding the graph.
     * @returns Self for chaining.
     */
    RealRenderer.prototype.reset = function () {
        this.strokes = [
            blankGraph_1.getBlankGraphPaths(this.dimensions, this.bgColor, this.bgType)
        ];
        this._strokeIndex = 0;
        this.resetTime();
        this._display(this.strokes[this._strokeIndex]);
        return this;
    };
    /**
     * Changes the background of the graph.
     * @param newBG New background.
     * @returns Self for chaining.
     */
    RealRenderer.prototype.changeBackground = function (newBG) {
        this.bgType = newBG;
        var newBGStrokes = blankGraph_1.getBlankGraphPaths(this.dimensions, this.bgColor, this.bgType);
        switch (this.bgType.type) {
            case 'axes':
                this.bgType.xOffset = clamp_1.clamp(this.bgType.xOffset, 0, 100); // %age
                this.bgType.yOffset = clamp_1.clamp(this.bgType.yOffset, 0, 100); // %age
                break;
            case 'grid':
                this.bgType.xSpacing = clamp_1.clamp(this.bgType.xSpacing, 0, 100); // %age
                this.bgType.ySpacing = clamp_1.clamp(this.bgType.ySpacing, 0, 100); // %age
                if (this.bgType.xSpacing === 0)
                    this.bgType.xSpacing = 1;
                if (this.bgType.ySpacing === 0)
                    this.bgType.ySpacing = 1;
                break;
            case 'ruled':
                this.bgType.spacing = clamp_1.clamp(this.bgType.spacing, 0, 100);
                if (this.bgType.spacing === 0)
                    this.bgType.spacing = 1;
                break;
            case 'none':
                break;
            default:
                break;
        }
        this.strokes[0].forEach(function (stroke) { return stroke.delete(); });
        this.strokes[0] = newBGStrokes;
        this._display(this.strokes[0]);
        return this;
    };
    return RealRenderer;
}(eventEmitter_1.EventEmitter));
exports.RealRenderer = RealRenderer;
