import { Path } from '../renderers/RealRenderer/strokeNodes/_path';
import { Text } from '../renderers/RealRenderer/strokeNodes/_text';
import { Circle } from '../renderers/RealRenderer/strokeNodes/_circle';
import { Polygon } from '../renderers/RealRenderer/strokeNodes/_polygon';
import { GroupNode } from '../renderers/RealRenderer/strokeNodes/_group';
/**
 * Dimensions of the graph in the format [width, height].
 */
export declare type GraphDimensions = [number, number];
/**
 * An array of R, G and B colors ranging between 0 and 1.
 */
export declare type Color = [number, number, number];
/**
 * A coordinate in the format [x, y].
 */
export declare type Coordinate = [number, number];
export declare type StrokeNode = Path | Circle | Text | Polygon | GroupNode;
export declare type SVGSection = 'bg' | 'strokes' | 'overlay';
export declare type SVGSections = {
    [Property in SVGSection]: SVGGElement;
};
export declare type StrokeNodeType = 'path' | 'circle' | 'text' | 'polygon' | 'group';
/**
 * One stroke is an combination of multiple stroke nodes
 */
export declare type Stroke = StrokeNode[];
export declare type StrokeNodeData = {
    type: StrokeNodeType;
    data: string;
    section: SVGSection;
};
export declare type StrokeExport = StrokeNodeData[];
/**
 * Data format for graph data exported  by Real Renderers
 */
export declare type RealExport = {
    exportData: StrokeExport[];
    strokeIndex: number;
    bgType: BGType;
    dimensions: GraphDimensions;
};
/**
 * Backround with axes.
 */
export declare type BGAxes = {
    type: 'axes';
    axesColor: Color;
    /**
     * %age offset of the y-axis (in the x-direction).
     */
    xOffset: number;
    /**
     * %age offset of the x-axis (in the y-direction).
     */
    yOffset: number;
};
/**
 * Ruled background.
 */
export declare type BGRuled = {
    type: 'ruled';
    orientation: 'vertical' | 'horizontal';
    /**
     * Percentage spacing between the ruled lines.
     */
    spacing: number;
    lineColor: Color;
};
/**
 * Background with grid.
 */
export declare type BGGrid = {
    type: 'grid';
    /**
     * Percentage spacing between the vertical grid lines lines.
     */
    xSpacing: number;
    /**
     * Percentage spacing between the horizontal grid lines lines.
     */
    ySpacing: number;
    lineColor: Color;
};
/**
 * Solid background.
 */
export declare type BGNone = {
    type: 'none';
};
export declare type BGType = BGNone | BGRuled | BGGrid | BGAxes;
export interface IRealRendererOptionals {
    /**
     * Background color.
     */
    bgColor: Color;
    /**
     * Type of the background.
     */
    bgType: BGType;
    /**
     * Number of updates to be made to the graph per frame.
     */
    drawsPerFrame: number;
    /**
     * Amount to increment the internal time by each frame.
     */
    timeStep: number;
    /**
     * Initial value of the internal time.
     */
    initTime: number;
    /**
     * Scaling of the graph. Larger number zooms.
     */
    scaleFactor: number;
}
export interface IRealRendererNonOptionals {
}
export declare type RealRendererSettings = IRealRendererNonOptionals & IRealRendererOptionals;
export declare type RealRendererOptions = IRealRendererNonOptionals & (IRealRendererOptionals | {});
export declare type RealRendererParameters = (IRealRendererOptionals | {});
